@@ -52,7 +52,7 @@ create table sessiondata (
 
   sessionid bigint not null,
   name varchar(50) default '' not null,
-  value varchar(255) default '' not null,
+  value text default '' not null,
   primary key (pkey)
 );
 
